@@ -10,6 +10,10 @@
 #   10-Apr-2013   jdw add convenience method getValueOrDefault()
 #   20-Jul-2015   jdw add selectIndicesFromList()
 #   01-Aug-2017   jdw migrate portions to public repo
+#    4-Oct-2018   jdw add optional method parameter returnCount=0 to selectValuesWhere() and selectValueListWhere()
+#   11-Nov-2018   jdw update consistent handling of raiseExceptions flag.
+#    5-May-2019   jdw add selectValuesWhereConditions() and countValuesWhereConditions()
+#    7-Aug-2019   jdw don't raise exception for *OrDefault() methods.
 ##
 """
 
@@ -21,9 +25,10 @@ from __future__ import absolute_import
 import logging
 import sys
 
-from mmcif.api.DataCategoryBase import DataCategoryBase
+from six.moves import range
+from six.moves import zip
 
-from six.moves import range, zip
+from mmcif.api.DataCategoryBase import DataCategoryBase
 
 __docformat__ = "restructuredtext en"
 __author__ = "John Westbrook"
@@ -42,10 +47,11 @@ class DataCategory(DataCategoryBase):
         """Summary
 
         Args:
-            name (TYPE): Description
-            attributeNameList (None, optional): Description
-            rowList (None, optional): Description
-            raiseExceptions (bool, optional): Description
+            name (str): Category name
+            attributeNameList (None, optional):  Initial attribute names
+            rowList (None, optional): Initial category data organized in rows corresponding to the attribute name list
+            raiseExceptions (bool, optional): Flag to control if expections are raised or handled internally
+            copyInputData (bool, optional):  Copy rather than reference input data
         """
         super(DataCategory, self).__init__(name, attributeNameList, rowList, raiseExceptions=raiseExceptions, copyInputData=copyInputData)
         #
@@ -54,8 +60,8 @@ class DataCategory(DataCategoryBase):
         self.__currentAttribute = None
         #
 
-    def setVerboseMode(self, bool):
-        self.__verbose = bool
+    def setVerboseMode(self, boolVal):
+        self.__verbose = boolVal
 
     def getCurrentAttribute(self):
         return self.__currentAttribute
@@ -67,12 +73,13 @@ class DataCategory(DataCategoryBase):
         """ Return a full row based on the length of the the attribute list or a row initialized with missing values
         """
         try:
-            if (len(self.data[index]) < self._numAttributes):
-                for ii in range(self._numAttributes - len(self.data[index])):
-                    self.data[index].append('?')
+            if len(self.data[index]) < self._numAttributes:
+                for _ in range(self._numAttributes - len(self.data[index])):
+                    self.data[index].append("?")
             return self.data[index]
         except Exception as e:
-            return ['?' for ii in range(self._numAttributes)]
+            logger.debug("Returning an empty row at %d (%s)", index, str(e))
+        return ["?" for ii in range(self._numAttributes)]
 
     def getAttributeListWithOrder(self):
         oL = []
@@ -86,12 +93,12 @@ class DataCategory(DataCategoryBase):
             i = self._attributeNameList.index(self._catalog[attributeNameLC])
             self._attributeNameList[i] = attributeName
             self._catalog[attributeNameLC] = attributeName
-            logger.info("Appending existing attribute %s\n" % attributeName)
+            logger.info("Appending existing attribute %s", attributeName)
         else:
             self._attributeNameList.append(attributeName)
             self._catalog[attributeNameLC] = attributeName
             # add a placeholder to any existing rows for the new attribute.
-            if (len(self.data) > 0):
+            if self.data:
                 for row in self.data:
                     row.append(defaultValue)
             #
@@ -108,14 +115,18 @@ class DataCategory(DataCategoryBase):
         else:
             rowI = rowIndex
 
-        if isinstance(attribute, self._string_types) and isinstance(rowI, int):
+        if isinstance(attribute, self._stringTypes) and isinstance(rowI, int):
             try:
                 return self.data[rowI][self._attributeNameList.index(attribute)]
-            except (IndexError):
-                raise IndexError
-        raise IndexError(attribute)
+            except IndexError:
+                if self._raiseExceptions:
+                    raise IndexError
+        if self._raiseExceptions:
+            raise IndexError(attribute)
+        else:
+            return None
 
-    def getValueOrDefault(self, attributeName=None, rowIndex=None, defaultValue=''):
+    def getValueOrDefault(self, attributeName=None, rowIndex=None, defaultValue=""):
         """  Within the current category return the value of input attribute in the input rowIndex [0-based].
 
              On error or if the value missing or null return the default value. Empty values returned as is.
@@ -131,22 +142,25 @@ class DataCategory(DataCategoryBase):
         else:
             rowI = rowIndex
 
-        if isinstance(attribute, self._string_types) and isinstance(rowI, int):
+        if isinstance(attribute, self._stringTypes) and isinstance(rowI, int):
             try:
                 tV = self.data[rowI][self._attributeNameList.index(attribute)]
-                if ((tV is None) or (tV in ['.', '?'])):
+                if (tV is None) or (tV in [".", "?"]):
                     return defaultValue
                 else:
                     return tV
             except Exception as e:
-                # logger.exception("Failing attributeName %s rowIndex %r defaultValue %r" % (attributeName, rowIndex, defaultValue))
-                raise e
+                logger.debug("Failing attributeName %s rowIndex %r defaultValue %r with %s", attributeName, rowIndex, defaultValue, str(e))
+                # if self._raiseExceptions:
+                #     raise e
+                # Returning default -- no exception
         else:
-            raise ValueError
+            if self._raiseExceptions:
+                raise ValueError
         #
         return defaultValue
 
-    def getFirstValueOrDefault(self, attributeNameList, rowIndex=0, defaultValue=''):
+    def getFirstValueOrDefault(self, attributeNameList, rowIndex=0, defaultValue=""):
         """ Return the value from the first non-null attribute found in the input attribute list
             from the row (rowIndex) in the current category object.
         """
@@ -154,13 +168,14 @@ class DataCategory(DataCategoryBase):
             for at in attributeNameList:
                 if self.hasAttribute(at):
                     tV = self.getValue(at, rowIndex)
-                    if ((tV is None) or (tV in ['', '.', '?'])):
+                    if (tV is None) or (tV in ["", ".", "?"]):
                         continue
                     else:
                         return tV
         except Exception as e:
-            raise e
-
+            logger.debug("Failing with %s", str(e))
+            # if self._raiseExceptions:
+            #    raise e
         return defaultValue
 
     def setValue(self, value, attributeName=None, rowIndex=None):
@@ -177,7 +192,7 @@ class DataCategory(DataCategoryBase):
         else:
             rowI = rowIndex
 
-        if isinstance(attribute, self._string_types) and isinstance(rowI, int) and (rowI >= 0):
+        if isinstance(attribute, self._stringTypes) and isinstance(rowI, int) and (rowI >= 0):
             try:
                 ind = -2
                 # if row index is out of range - add the rows -
@@ -188,27 +203,36 @@ class DataCategory(DataCategoryBase):
                 ind = self._attributeNameList.index(attribute)
 
                 # extend the list if needed -
-                if (ind >= ll):
+                if ind >= ll:
                     self.data[rowI].extend([None for ii in range(ind - (ll - 1))])
 
                 self.data[rowI][ind] = value
                 return True
-            except (IndexError):
+            except IndexError:
                 if self.__verbose:
-                    logger.exception("DataCategory(setvalue) index error category %s attribute %s row index %d col %d rowlen %d value %r\n" %
-                                     (self._name, attribute, rowI, ind, len(self.data[rowI]), value))
-                    logger.debug("DataCategory(setvalue) attribute %r length attribute list %d \n" % (attribute, len(self._attributeNameList)))
-                    for ii, a in enumerate(self._attributeNameList):
-                        logger.debug("DataCategory(setvalue) %d attributeName %r\n" % (ii, a))
-
-                raise IndexError
-            except (ValueError):
+                    logger.exception(
+                        "DataCategory(setvalue) index error category %s attribute %s row index %d col %d rowlen %d value %r",
+                        self._name,
+                        attribute,
+                        rowI,
+                        ind,
+                        len(self.data[rowI]),
+                        value,
+                    )
+                    logger.debug("DataCategory(setvalue) attribute %r length attribute list %d", attribute, len(self._attributeNameList))
+                    for ii, aV in enumerate(self._attributeNameList):
+                        logger.debug("DataCategory(setvalue) %d attributeName %r", ii, aV)
+                if self._raiseExceptions:
+                    raise IndexError
+            except ValueError:
                 if self.__verbose:
-                    logger.exception("DataCategory(setvalue) value error category %s attribute %s row index %d value %r\n" %
-                                     (self._name, attribute, rowI, value))
-                raise ValueError
+                    logger.exception("DataCategory(setvalue) value error category %s attribute %s row index %d value %r", self._name, attribute, rowI, value)
+                if self._raiseExceptions:
+                    raise ValueError
         else:
-            raise ValueError
+            if self._raiseExceptions:
+                raise ValueError
+        return False
 
     def __emptyRow(self):
         return [None for ii in range(len(self._attributeNameList))]
@@ -225,7 +249,9 @@ class DataCategory(DataCategoryBase):
                     numReplace += 1
             return numReplace
         except Exception as e:
-            raise e
+            if self._raiseExceptions:
+                raise e
+        return numReplace
 
     def replaceSubstring(self, oldValue, newValue, attributeName):
         try:
@@ -240,7 +266,9 @@ class DataCategory(DataCategoryBase):
                     numReplace += 1
             return numReplace
         except Exception as e:
-            raise e
+            if self._raiseExceptions:
+                raise e
+        return numReplace
 
     def selectIndices(self, attributeValue, attributeName):
         try:
@@ -253,7 +281,9 @@ class DataCategory(DataCategoryBase):
                     rL.append(ii)
             return rL
         except Exception as e:
-            raise e
+            if self._raiseExceptions:
+                raise e
+        return rL
 
     def selectIndicesFromList(self, attributeValueList, attributeNameList):
         rL = []
@@ -273,73 +303,249 @@ class DataCategory(DataCategoryBase):
                     rL.append(ii)
         except Exception as e:
             if self.__verbose:
-                logger.exception("Selection/index failure for values %r" % attributeValueList)
-            raise e
+                logger.exception("Selection/index failure for values %r", attributeValueList)
+            if self._raiseExceptions:
+                raise e
 
         return rL
 
-    def selectValuesWhere(self, attributeName, attributeValueWhere, attributeNameWhere):
+    def selectValuesWhere(self, attributeName, attributeValueWhere, attributeNameWhere, returnCount=0):
         rL = []
         try:
+            iCount = 0
             ind = self._attributeNameList.index(attributeName)
             indWhere = self._attributeNameList.index(attributeNameWhere)
-            for ii, row in enumerate(self.data):
+            for row in self.data:
                 if attributeValueWhere == row[indWhere]:
                     rL.append(row[ind])
+                    iCount += 1
+                    if returnCount and (iCount >= returnCount):
+                        break
         except Exception as e:
             if self.__verbose:
                 logger.exception("Selection failure")
-            raise e
+            if self._raiseExceptions:
+                raise e
         return rL
 
-    def selectValueListWhere(self, attributeNameList, attributeValueWhere, attributeNameWhere):
+    def selectValueListWhere(self, attributeNameList, attributeValueWhere, attributeNameWhere, returnCount=0):
         """ Return a  list of lists containing the values of input attributeNameList
             satisfiying the attribute value where condition.
         """
         rL = []
         try:
+            iCount = 0
             indList = []
             for at in attributeNameList:
                 indList.append(self._attributeNameList.index(at))
             indWhere = self._attributeNameList.index(attributeNameWhere)
-            for ii, row in enumerate(self.data):
+            for row in self.data:
                 if attributeValueWhere == row[indWhere]:
                     rL.append([row[jj] for jj in indList])
+                    iCount += 1
+                    if returnCount and (iCount >= returnCount):
+                        break
         except Exception as e:
             if self.__verbose:
                 logger.exception("Selection failure")
-            raise e
+            if self._raiseExceptions:
+                raise e
         return rL
 
-    def invokeAttributeMethod(self, attributeName, type, method, db):
+    def selectValuesWhereConditions(self, attributeName, conditionsD, returnCount=0):
+        rL = []
+        try:
+            iCount = 0
+            ind = self._attributeNameList.index(attributeName)
+            idxD = {k: self._attributeNameList.index(k) for k, v in conditionsD.items()}
+            #
+            #
+            for row in self.data:
+                ok = True
+                for k, v in conditionsD.items():
+                    ok = (v == row[idxD[k]]) and ok
+                if ok:
+                    rL.append(row[ind])
+                    iCount += 1
+                    if returnCount and (iCount >= returnCount):
+                        break
+        except Exception as e:
+            if self.__verbose:
+                logger.exception("Selection failure")
+            if self._raiseExceptions:
+                raise e
+        return rL
+
+    def countValuesWhereConditions(self, conditionsD):
+        """Count row instances subject to input equivalence conditions
+
+        Args:
+            conditionsD (dict): {'atName': value, ....}
+
+        Raises:
+            e: any failure
+
+        Returns:
+            int: count of instancese satisfying input conditions
+        """
+        try:
+            iCount = 0
+            idxD = {k: self._attributeNameList.index(k) for k, v in conditionsD.items()}
+            #
+            for row in self.data:
+                ok = True
+                for k, v in conditionsD.items():
+                    ok = (v == row[idxD[k]]) and ok
+                if ok:
+                    iCount += 1
+
+        except Exception as e:
+            if self.__verbose:
+                logger.exception("Selection failure")
+            if self._raiseExceptions:
+                raise e
+        return iCount
+
+    def countValuesWhereOpConditions(self, conditionTupleList):
+        """Count row instances subject to input condition list
+
+        Args:
+            conditionTupleList (list): (attributeName, op, value) where (op = 'eq', 'gt(int)', 'lt(int)', 'in', 'ne', 'not in')
+
+        Raises:
+            e: any failure
+
+        Returns:
+            int: count of instancese satisfying input conditions
+        """
+        try:
+            iCount = 0
+            idxD = {atName: self._attributeNameList.index(atName) for (atName, op, value) in conditionTupleList}
+            #
+            for row in self.data:
+                ok = True
+                for (atName, op, v) in conditionTupleList:
+                    if op == "eq":
+                        ok = (v == row[idxD[atName]]) and ok
+                    elif op == "ne":
+                        ok = (v != row[idxD[atName]]) and ok
+                    elif op == "lt(int)":
+                        ok = (int(row[idxD[atName]]) < v) and ok
+                    elif op == "gt(int)":
+                        ok = (int(row[idxD[atName]]) > v) and ok
+                    elif op == "in":
+                        ok = (row[idxD[atName]] in v) and ok
+                    elif op == "not in":
+                        ok = (row[idxD[atName]] not in v) and ok
+                if ok:
+                    iCount += 1
+
+        except Exception as e:
+            if self.__verbose:
+                logger.exception("Selection failure")
+            if self._raiseExceptions:
+                raise e
+        return iCount
+
+    #
+    def getCombinationCounts(self, attributeList):
+        """ Count the value occurences of the input attributeList in the category.
+
+            Args:
+                attributeList (list): target list of attribute names
+
+            Returns:
+
+                cD[(attribute value, ... )] = count
+
+        """
+        cD = {}
+        try:
+            idxL = [self._attributeNameList.index(atName) for atName in attributeList]
+            #
+            for row in self.data:
+                ky = tuple([row[jj] for jj in idxL])
+                cD[ky] = cD[ky] + 1 if ky in cD else 1
+        except Exception as e:
+            if self.__verbose:
+                logger.exception("Selection failure")
+            if self._raiseExceptions:
+                raise e
+        return cD
+
+    def getCombinationCountsWithConditions(self, attributeList, conditionTupleList):
+        """ Count the value occurences of the input attributeList in the category.
+
+            Args:
+                attributeList (list): target list of attribute names
+                conditionTupleList (list): (attributeName, op, value) where (op = 'eq', 'gt(int)', 'lt(int)', 'in', 'ne', 'not in')
+
+            Returns:
+
+                cD[(attribute value, ... )] = count
+        """
+        cD = {}
+        try:
+            idxL = [self._attributeNameList.index(atName) for atName in attributeList]
+            idxD = {atName: self._attributeNameList.index(atName) for (atName, op, value) in conditionTupleList}
+            #
+            for row in self.data:
+                ok = True
+                for (atName, op, v) in conditionTupleList:
+                    if op == "eq":
+                        ok = (v == row[idxD[atName]]) and ok
+                    elif op == "ne":
+                        ok = (v != row[idxD[atName]]) and ok
+                    elif op == "lt(int)":
+                        ok = (int(row[idxD[atName]]) < v) and ok
+                    elif op == "gt(int)":
+                        ok = (int(row[idxD[atName]]) > v) and ok
+                    elif op == "in":
+                        ok = (row[idxD[atName]] in v) and ok
+                    elif op == "not in":
+                        ok = (row[idxD[atName]] not in v) and ok
+                if ok:
+                    ky = tuple([row[jj] for jj in idxL])
+                    cD[ky] = cD[ky] + 1 if ky in cD else 1
+        except Exception as e:
+            if self.__verbose:
+                logger.exception("Selection failure")
+            if self._raiseExceptions:
+                raise e
+        return cD
+
+    def invokeAttributeMethod(self, attributeName, mType, method, db):
+        _ = mType
+        _ = db
         self._currentRowIndex = 0
         self.__currentAttribute = attributeName
         self.appendAttribute(attributeName)
-        currentRowIndex = self._currentRowIndex
+        currentRowIndex = self._currentRowIndex  # pylint: disable=possibly-unused-variable
         #
         ind = self._attributeNameList.index(attributeName)
-        if len(self.data) == 0:
+        if not self.data:
             row = [None for ii in range(len(self._attributeNameList) * 2)]
             row[ind] = None
             self.data.append(row)
 
         for row in self.data:
             ll = len(row)
-            if (ind >= ll):
+            if ind >= ll:
                 row.extend([None for ii in range(2 * ind - ll)])
                 row[ind] = None
-            exec(method.getInline(), globals(), locals())
+            exec(method.getInline(), globals(), locals())  # pylint: disable=exec-used
             self._currentRowIndex += 1
             currentRowIndex = self._currentRowIndex
 
-    def invokeCategoryMethod(self, type, method, db):
+    def invokeCategoryMethod(self, mType, method, db):
+        _ = mType
+        _ = db
         self._currentRowIndex = 0
-        exec(method.getInline(), globals(), locals())
+        exec(method.getInline(), globals(), locals())  # pylint: disable=exec-used
 
     def printIt(self, fh=sys.stdout):
         fh.write("--------------------------------------------\n")
-        fh.write("  Category: %s attribute list length: %d\n" %
-                 (self._name, len(self._attributeNameList)))
+        fh.write("  Category: %s attribute list length: %d\n" % (self._name, len(self._attributeNameList)))
         for at in self._attributeNameList:
             fh.write("  Category: %s attribute: %s\n" % (self._name, at))
 
@@ -349,21 +555,21 @@ class DataCategory(DataCategoryBase):
             #
             if len(row) == len(self._attributeNameList):
                 for ii, v in enumerate(row):
-                    fh.write("        %30s: %s ...\n" % (self._attributeNameList[ii], str(v)[:30]))
+                    fh.write("       %30s: %s ...\n" % (self._attributeNameList[ii], str(v)[:30]))
             else:
-                fh.write("+WARNING - %s data length %d attribute name length %s mismatched\n" %
-                         (self._name, len(row), len(self._attributeNameList)))
+                fh.write("+WARNING - %s data length %d attribute name length %s mismatched\n" % (self._name, len(row), len(self._attributeNameList)))
 
     def dumpIt(self, fh=sys.stdout):
         fh.write("--------------------------------------------\n")
-        fh.write("  Category: %s attribute list length: %d\n" %
-                 (self._name, len(self._attributeNameList)))
+        fh.write("  Category: %s attribute list length: %d\n" % (self._name, len(self._attributeNameList)))
         for at in self._attributeNameList:
             fh.write("  Category: %s attribute: %s\n" % (self._name, at))
 
         fh.write("  Value list length: %d\n" % len(self.data))
-        for row in self.data:
+        for jj, row in enumerate(self.data):
             for ii, v in enumerate(row):
-                fh.write("        %30s: %s\n" % (self._attributeNameList[ii], v))
+                fh.write("%4d        %30s: %s\n" % (jj, self._attributeNameList[ii], v))
+
+
 ##
 ##
